// -*-c++-*-
// 
// blasvis++ - C++ version of graphics routines
//
// Blasvis : Main class header file
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>

#ifndef _BLASVIS_H
#define _BLASVIS_H

#include "dataset.h"
#include "datavect.h"
#include "blasfile.h"
#include "models.h"
#include "../config.h"

class Blasvis {

public:
  Blasvis();
  ~Blasvis();

  int Run(int, char**);
  void MainMenu();
  void InputFileOpts();
  void GetCommandFile();
  void SetInterpolationGrid();
  void SelectData();
  void SelectOutputDev();
  void SetRange();
  void SetContours();
  void SetTitles();
  void SetPlotOpts();
  void StartPlot();
  void NextPage();
  void PlotData();
  void PlotTimeSeries();
  void PlotMisc();
  void FinishPlot();
  void OutputData();
  void OutputTimeSeries();
  void OutputDataStats();
  void SetColScheme();
  void CheckSeparation();
  void GetDrag();
  void DerivData();
  void NormData();
  void SetVariable();
  void GetValues();

private:
  bool GetYesNo();
  bldata GetReal(std::istream * = NULL, char delim='\n');
  int GetInt(std::istream * = NULL, char delim='\n');
  int GetDimOpt(int,bool);
  int GetRunNumber(char*);
  int GetTime(char*);
  char* BaseName(char*);
  int StringExpand(char*, char*, int); 
  void PlotSingleVectors(bool);
  void PlotLabels();

  void LoadGrids();

  // Variables / object definitions
private:
  ReadFile *datafile;
  DataPlot plot;

  // Keep the basic data sets loaded in.
  // Strictly this is not necessary as ReadFile caches them.
  DataSet *temperature;

  // Input stream for program
  // This can be a file or cin.
  std::istream *input;
  std::ifstream finput;
  bool isCmdFile;

  // Input stream for program
  // This can be a file or cout or /dev/null.
  std::ostream *output;
  std::ofstream foutput;
  
  DataSet *data1, *data2, *data3;
  DataVect *vect;
  TimeSeries *tdata1, *tdata2;

  // Run number and time from file name
  int runno,time;

  // Verbose command mode prints out the menu options when reading cmd files
  bool verbose;

  // Values of blasvis++ variables
  bldata varvals[10];

  static const char version[];

  // Type of model being plotted.
  modeltype model;
 
  // Name of the program
  char *progname;

  // Use LES averaged statistics if available
  bool useLES;
};

const char Blasvis::version[] = VERSION;

#endif
