//
// xblasvis - C++ version of graphics routines - X11 GUI 
//
// PlotDlg : Dialog box to plot data
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

#include "exportdlg.h"

using namespace std;

//
// Constructor for class
//
ExportDlg::ExportDlg(QWidget *parent, ReadFile *dfile, OptDlg *o, const char *name) : QDialog(parent,name, TRUE) {

  QString text;
  HeaderItem *item;

  // Set up the data file and plot object 
  datafile = dfile;
  optDlg = o;

  // Set up the window
  resize(400,300);

  // Set up listbox with data
  if (datafile != NULL) 
    item = datafile->GetFirstHeaderItem();
  else 
    item = NULL;  

  dataBox = new QListBox(this);
  while (item != NULL) {
    if (item->ndim > 0) {
      dataBox->insertItem(item->name);
    }
    item = item->next;    
  }
  
  dataBox->setGeometry(20,50,150,100);

  QLabel *label = new QLabel("Data set",this);
  label->setGeometry(20,20,150,30);

  // Create edit box for file name
  label = new QLabel("File name",this);
  label->setGeometry(20,160,150,30);
  fnameEdit = new QLineEdit(this);
  fnameEdit->setGeometry(20,190,150,30);

  // Create button to fire up save file dialog
  QPushButton *browseButton = new QPushButton("Browse",this);
  browseButton->setGeometry(190, 110, 100,30);
  connect(browseButton, SIGNAL(clicked()), this, SLOT(namePressed()) );

  QPushButton *okButton = new QPushButton("OK",this);
  okButton->setGeometry(50,250,100,30);
  QPushButton *cancelButton = new QPushButton("Cancel",this);
  cancelButton->setGeometry(200,250,100,30);

  connect(okButton, SIGNAL(clicked()), this, SLOT(apply()) );
  connect(cancelButton, SIGNAL(clicked()), this,SLOT(reject()) );
}


//
// Slot called when Filename is pressed
//
void ExportDlg::namePressed() {
  QString f;

  f = QFileDialog::getSaveFileName(fname);
  
  if (!f.isEmpty()) {
    fname = f;
    fnameEdit->setText(fname);
  }
}


//
// Slot called when button to set options is pressed
//
void ExportDlg::optPressed() {
  optDlg->exec();
}

//
// Called to check and update the options
//
void ExportDlg::apply() {

  int id;
  DataSet *data;

  // Get the data name if there is an item selected
  id = dataBox->currentItem();
  if (id == -1) {
    QMessageBox::warning(this,"Warning!","No data set selected");
    return;
  }

  fname = fnameEdit->text();

  data = datafile->GetData(dataBox->text(id));
  data->DisplayData(fname);

  accept();
  
}

#include "exportdlg.moc"
