//
// xblasvis - C++ version of graphics routines - X11 GUI 
//
// GUI : Main GUI window
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#ifndef _GUI_H
#define _GUI_H

#include <qmainwindow.h>
#include <qmenubar.h>
#include <qkeycode.h>

#include "readfile.h"
#include "dataplot.h"
#include "dataset.h"
#include "models.h"
#include "plotdlg.h"
#include "plot1ddlg.h"
#include "optdlg.h"
#include "exportdlg.h"

class GUI : public QMainWindow {
  Q_OBJECT
    public:
  GUI(QWidget *, const char * = NULL);
  ~GUI();

  // Private functions to handle menu requests
 public slots:
 void openData();
  void swapByteOrder();
  void openCmd();
  void saveData();
  void closeData();
  void selectData();
  void setPlotOptions();
  void plotData();
  void plot1dData();
  void displayStats();
  void about();
  void setDevice(int);
  void startPlot();
  void finishPlot();
  void nextPage();

  // Private data 
 private:
  // GUI data
 // Main menubar
  QMenuBar *menu;

  // Popup menus we need to keep track of
  QPopupMenu *devMenu;
  QPopupMenu *fileMenu;

  // Dialog boxes
  PlotDlg *plotDlg;
  Plot1dDlg *plot1dDlg;
  OptDlg *optDlg;
  ExportDlg *expDlg;

  // Other data
  ReadFile *datafile;
  DataPlot plot;

  // Keep track of the temperature field
  DataSet *temperature;

  // Current data loaded
  DataSet *data1, *data2, *data3;
  DataVect *vect;

  // Type of model being plotted.
  modeltype model;

  // Run number and time from file name
  int runno,time;

  // Current output device
  DataPlot::devtype dev;

  // Whether we are swapping endian or not and menu id
  bool swapEndian;
  int swapEndianId;

};

#endif
