//
// xblasvis - C++ version of graphics routines - X11 GUI 
//
// PlotDlg : Dialog box to plot data
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

#include "optdlg.h"

using namespace std;

//
// Constructor for class
//
OptDlg::OptDlg(QWidget *parent, DataPlot *p, const char *name) : QTabDialog(parent,name, TRUE) {

  QString text;

  // Set up the data plot
  plot = p;

  // Set up the window
  resize(700,450);

  // Set default options
  title = "Data Plot - %d. Run %r, time %ts.";
  xtitle = "%x";
  ytitle = "%y";
  dtitle = "%d";

  xmin = 0;
  xmax = 0;
  ymin = 0;
  ymax = 0;
  zmin = 0;
  zmax = 0;
  contmin = 0;
  contmax = 0;
  ncont = 30;
  xInterp = 100;
  yInterp = 100;
  xscale = 1;
  yscale = 1;
  zscale = 1;

  legend = TRUE;
  hillBase = TRUE;
  hillIntersect = TRUE;
  invertHoriz = FALSE;
  mesh = TRUE;
  labelCont = FALSE;

  colScheme = 0;

  // Create the tabs
  QWidget *titlePane = new QWidget(this, "title_tab");
  titlePane->resize(650,550);

  // Set up the title pane
  QLabel *label = new QLabel("Plot title",titlePane);
  label->setGeometry(20,10,150,30);
  titleEdit = new QLineEdit(titlePane);
  titleEdit->setText(title);
  titleEdit->setGeometry(20,40,150,30);

  label = new QLabel("X axis title",titlePane);
  label->setGeometry(20,80,150,30);
  xtitleEdit = new QLineEdit(titlePane);
  xtitleEdit->setText(xtitle);
  xtitleEdit->setGeometry(20,110,150,30);

  label = new QLabel("Y axis title",titlePane);
  label->setGeometry(200,80,150,30);
  ytitleEdit = new QLineEdit(titlePane);
  ytitleEdit->setText(ytitle);
  ytitleEdit->setGeometry(200,110,150,30);

  label = new QLabel("Data title",titlePane);
  label->setGeometry(200,10,150,30);
  dtitleEdit = new QLineEdit(titlePane);
  dtitleEdit->setText(dtitle);
  dtitleEdit->setGeometry(200,40,150,30);

  // Set up the range pane
  QWidget *rangePane = new QWidget(this, "range_tab");
  rangePane->resize(650,700);

  QGroupBox *group = new QGroupBox("X-axis",rangePane);
  group->setGeometry(10,10,650,90);
  xAutoCheck = new QCheckBox("Auto range", group);
  xAutoCheck->setGeometry(10,30,100,30);
  xAutoCheck->setChecked(xmin == xmax);

  label = new QLabel("Minimum",group);
  label->setGeometry(120,20,150,30);
  xminEdit = new QLineEdit(group);
  text.setNum(xmin);
  xminEdit->setText(text);
  xminEdit->setGeometry(120,50,150,30);

  label = new QLabel("Maximum",group);
  label->setGeometry(300,20,150,30);
  xmaxEdit = new QLineEdit(group);
  text.setNum(xmax);
  xmaxEdit->setText(text);
  xmaxEdit->setGeometry(300,50,150,30);

  label = new QLabel("Scaling",group);
  label->setGeometry(480,20,150,30);
  xscaleEdit = new QLineEdit(group);
  text.setNum(xscale);
  xscaleEdit->setText(text);
  xscaleEdit->setGeometry(480,50,150,30);

  group = new QGroupBox("Y-axis",rangePane);
  group->setGeometry(10,120,650,90);
  yAutoCheck = new QCheckBox("Auto range", group);
  yAutoCheck->setGeometry(10,30,100,30);
  yAutoCheck->setChecked(ymin == ymax);

  label = new QLabel("Minimum",group);
  label->setGeometry(120,20,150,30);
  yminEdit = new QLineEdit(group);
  text.setNum(ymin);
  yminEdit->setText(text);
  yminEdit->setGeometry(120,50,150,30);

  label = new QLabel("Maximum",group);
  label->setGeometry(300,20,150,30);
  ymaxEdit = new QLineEdit(group);
  text.setNum(ymax);
  ymaxEdit->setText(text);
  ymaxEdit->setGeometry(300,50,150,30);

  label = new QLabel("Scaling",group);
  label->setGeometry(480,20,150,30);
  yscaleEdit = new QLineEdit(group);
  text.setNum(yscale);
  yscaleEdit->setText(text);
  yscaleEdit->setGeometry(480,50,150,30);

  group = new QGroupBox("Z-axis",rangePane);
  group->setGeometry(10,230,650,90);
  zAutoCheck = new QCheckBox("Auto range", group);
  zAutoCheck->setGeometry(10,30,100,30);
  zAutoCheck->setChecked(zmin == zmax);

  label = new QLabel("Minimum",group);
  label->setGeometry(120,20,150,30);
  zminEdit = new QLineEdit(group);
  text.setNum(zmin);
  zminEdit->setText(text);
  zminEdit->setGeometry(120,50,150,30);

  label = new QLabel("Maximum",group);
  label->setGeometry(300,20,150,30);
  zmaxEdit = new QLineEdit(group);
  text.setNum(zmax);
  zmaxEdit->setText(text);
  zmaxEdit->setGeometry(300,50,150,30);

  label = new QLabel("Scaling",group);
  label->setGeometry(480,20,150,30);
  zscaleEdit = new QLineEdit(group);
  text.setNum(zscale);
  zscaleEdit->setText(text);
  zscaleEdit->setGeometry(480,50,150,30);

  // Set up contours / interpolation tab
  QWidget *interpPane = new QWidget(this, "range_tab");
  interpPane->resize(480,550);

  group = new QGroupBox("Contours",interpPane);
  group->setGeometry(10,10,650,90);
  contAutoCheck = new QCheckBox("Auto range", group);
  contAutoCheck->setGeometry(10,30,100,30);
  contAutoCheck->setChecked(contmin == contmax);

  label = new QLabel("Minimum",group);
  label->setGeometry(120,20,150,30);
  contminEdit = new QLineEdit(group);
  text.setNum(contmin);
  contminEdit->setText(text);
  contminEdit->setGeometry(120,50,150,30);

  label = new QLabel("Maximum",group);
  label->setGeometry(300,20,150,30);
  contmaxEdit = new QLineEdit(group);
  text.setNum(contmax);
  contmaxEdit->setText(text);
  contmaxEdit->setGeometry(300,50,150,30);

  label = new QLabel("Number of contours",group);
  label->setGeometry(480,20,150,30);
  ncontEdit = new QLineEdit(group);
  text.setNum(ncont);
  ncontEdit->setText(text);
  ncontEdit->setGeometry(480,50,150,30);

  group = new QGroupBox("Interpolation",interpPane);
  group->setGeometry(10,120,460,90);

  label = new QLabel("Points in x direction",group);
  label->setGeometry(20,20,150,30);
  xInterpEdit = new QLineEdit(group);
  text.setNum(xInterp);
  xInterpEdit->setText(text);
  xInterpEdit->setGeometry(20,50,150,30);

  label = new QLabel("Points in y direction",group);
  label->setGeometry(200,20,150,30);
  yInterpEdit = new QLineEdit(group);
  text.setNum(yInterp);
  yInterpEdit->setText(text);
  yInterpEdit->setGeometry(200,50,150,30);

  // Create the plot options pane
  QWidget *optPane = new QWidget(this, "opt_tab");
  optPane->resize(480,550);

  group = new QGroupBox("Plot additions",optPane);
  group->setGeometry(10,10,460,110);

  legendCheck = new QCheckBox("Legend", group);
  legendCheck->setGeometry(10,30,110,30);
  legendCheck->setChecked(legend);

  hillBaseCheck = new QCheckBox("Hill base", group);
  hillBaseCheck->setGeometry(130,30,110,30);
  hillBaseCheck->setChecked(hillBase);

  hillIntersectCheck = new QCheckBox("Hill intersection", group);
  hillIntersectCheck->setGeometry(250,30,110,30);
  hillIntersectCheck->setChecked(hillIntersect);
  
  meshCheck = new QCheckBox("Mesh", group);
  meshCheck->setGeometry(10,70,110,30);
  meshCheck->setChecked(mesh);

  labelContCheck = new QCheckBox("Label contours", group);
  labelContCheck->setGeometry(130,70,110,30);
  labelContCheck->setChecked(labelCont);

  label = new QLabel("Colour scheme",optPane);
  label->setGeometry(10,130,150,30);
  colourCombo = new QComboBox(FALSE, optPane, "colour combo");
  colourCombo->insertItem("Default");
  colourCombo->insertItem("Monochrome");
  colourCombo->insertItem("Colour");
  colourCombo->insertItem("Monochrome (zero white)");
  colourCombo->insertItem("Colour (zero white)");
  colourCombo->insertItem("Reversed monochrome");
  colourCombo->insertItem("Reversed colour");
  colourCombo->insertItem("Reversed monochrome (zero white)");
  colourCombo->insertItem("Reversed colour (zero white)");
  colourCombo->setGeometry(10,160,200,30);
  colourCombo->setCurrentItem(colScheme);
  
  // Add in the tabs
  addTab(titlePane, "Titles");
  addTab(rangePane, "Axes range");
  addTab(interpPane, "Contours / Interpolation");
  addTab(optPane, "Plot options");
  
  // Set up the buttons
  setOkButton();
  setApplyButton();
  setCancelButton();
  
  connect(this,SIGNAL(applyButtonPressed()), SLOT(apply()) );
    
}

//
// Called when OK or Apply is pressed to update the options
//
void OptDlg::apply() {
  const char *t, *xt, *yt, *dt;
  QString text;
  bool err1, err2, err3;
  bldata xval1, xval2, yval1, yval2, zval1, zval2, val1, val2, val3;
  int ival, ixval, iyval, ph;
  bool bval1, bval2;

  // Get titles and update if needed
  t = titleEdit->text();
  xt = xtitleEdit->text();
  yt = ytitleEdit->text();
  dt = dtitleEdit->text();

  if ( (title != t) || (xtitle != xt) || (ytitle != yt) || (dtitle != dt) ) {
    title = t;
    xtitle = xt;
    ytitle = yt;
    dtitle = dt;

    plot->SetTitles((char *)t,(char *)xt,(char *)yt,(char *)dt);
  }

  // Get ranges for axes
  if (xAutoCheck->isChecked()) {
    xval1 = 0;
    xval2 = 0;
  }
  else {
    text = xminEdit->text();
    xval1 = (bldata) text.toDouble(&err1);
    text = xmaxEdit->text();
    xval2 = (bldata) text.toDouble(&err2);
    
    if (!err1 || !err2) {
      QMessageBox::warning(this,"Warning!","Error in x range");
      xval1 = xmin;
      xval2 = xmax;
    }
  }

  if (yAutoCheck->isChecked()) {
    yval1 = 0;
    yval2 = 0;
  }
  else {
    text = yminEdit->text();
    yval1 = (bldata) text.toDouble(&err1);
    text = ymaxEdit->text();
    yval2 = (bldata) text.toDouble(&err2);

    if (!err1 || !err2) {
      QMessageBox::warning(this,"Warning!","Error in y range");
      yval1 = ymin;
      yval2 = ymax;
    }
  }

  if (zAutoCheck->isChecked()) {
    zval1 = 0;
    zval2 = 0;
  }
  else {
    text = zminEdit->text();
    zval1 = (bldata) text.toDouble(&err1);
    text = zmaxEdit->text();
    zval2 = (bldata) text.toDouble(&err2);

    if (!err1 || !err2) {
      QMessageBox::warning(this,"Warning!","Error in z range");
      zval1 = zmin;
      zval2 = zmax;
    }
  }

  if ( (xval1 != xmin) || (xval2 != xmax) || (yval1 != ymin) || (yval2 != ymax) || 
       (zval1 != zmin) || (zval2 != zmax) ) {
    
    plot->SetRange(xval1, xval2, yval1, yval2, zval1, zval2);

    xmin = val1;
    xmax = val2;
    ymin = yval1;
    ymax = yval2;
    zmin = zval1;
    zmax = zval2;
  }
  
  // Set up contours
  if (contAutoCheck->isChecked()) {
    val1 = 0;
    val2 = 0;
  }
  else {
    text = contminEdit->text();
    val1 = (bldata) text.toDouble(&err1);
    text = contmaxEdit->text();
    val2 = (bldata) text.toDouble(&err2);
    text = ncontEdit->text();
    ival = text.toInt(&err3);
    
    if (!err1 || !err2 || !err3 || (ival <= 0)) {
      QMessageBox::warning(this,"Warning!","Error in contour range");
      val1 = contmin;
      val2 = contmax;
      ival = ncont;
    }
  }
  if ( (val1 != contmin) || (val2 != contmax) || (ival != ncont) ) {
    plot->SetContours(val1, val2, ncont);
    contmin = val1;
    contmax = val2;
    ncont = ival;
  }

    
  // Set up interpolation 
  text = xInterpEdit->text();
  ixval = text.toInt(&err1);
  text = yInterpEdit->text();
  iyval = text.toInt(&err2);
 
  if (!err1 || !err2 || (ixval <= 0) || (iyval <= 0) ) {
      QMessageBox::warning(this,"Warning!","Error in interpolation settings");
  }
  else {
    xInterp = ixval;
    yInterp = iyval;
    plot->SetInterpolation(xInterp, yInterp);
  }

  // Get scaling factors for axes
  text = xscaleEdit->text();
  val1 = (bldata) text.toDouble(&err1);
  text = yscaleEdit->text();
  val2 = (bldata) text.toDouble(&err2);
  text = zscaleEdit->text();
  val3 = text.toInt(&err3);
  if (!err1 || !err2 || !err3) {
    QMessageBox::warning(this,"Warning!","Error in axes scaling factors");
  }
  else {
    if ( (val1 != xscale) || (val2 != yscale) || (val3 != zscale) ) {
      plot->SetAxisScaling(1,val1);
      plot->SetAxisScaling(2,val2);
      plot->SetAxisScaling(3,val3);
      xscale = val1;
      yscale = val2;
      zscale = val3;
    }
  }

  // Get boolean plot options
  bval1 = legendCheck->isChecked();
  if (bval1 != legend) {
    plot->SetPlotLegend(bval1);
    legend = bval1;
  }

  bval1 = hillBaseCheck->isChecked();
  bval2 = hillIntersectCheck->isChecked();
  if ( (bval1 != hillBase) || (bval2 != hillIntersect) ) {
    ph = 0;
    if (bval1) {
      ph |= DataPlot::PLOTBASE;
    }
    if (bval2) {
      ph |= DataPlot::PLOTINTERSECT;
    }
    plot->SetPlotHill(ph);
    hillBase = bval1;
    hillIntersect = bval2;
  }

  bval1 = meshCheck->isChecked();
  if (bval1 != mesh) {
    plot->SetDrawMesh(bval1);
    mesh = bval1;
  }

  bval1 = labelContCheck->isChecked();
  if (bval1 != labelCont) {
    plot->SetPlotContLabel(bval1);
    labelCont = bval1;
  }

  ival = colourCombo->currentItem();
  if (ival != colScheme) {
    plot->SetColScheme(ival);
    colScheme = ival;
  }
  
}

#include "optdlg.moc"
