// 
// xblasvis - C++ version of graphics routines - X11 GUI  
// 
// OptDlg : Dialog box for setting plot options
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#ifndef _OPTDLG_H
#define _OPTDLG_H

#include <qtabdialog.h>
#include <qlistbox.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>

#include "dataplot.h"


class OptDlg : public QTabDialog {
  Q_OBJECT
    public:
  OptDlg(QWidget *, DataPlot *, const char * = NULL);  
  
  public slots:
    void apply();
    
 private:
  
  // GUI widgets
  QWidget *titlePane;
  QWidget *rangePane;
  QWidget *interpPane;
  QWidget *optPane;
  QLineEdit *titleEdit, *xtitleEdit, *ytitleEdit, *dtitleEdit;  
  QLineEdit *xminEdit, *xmaxEdit, *yminEdit, *ymaxEdit, *zminEdit, *zmaxEdit;
  QLineEdit *xscaleEdit, *yscaleEdit, *zscaleEdit;
  QLineEdit *contminEdit, *contmaxEdit, *ncontEdit, *dataScaleEdit;
  QCheckBox *xAutoCheck, *yAutoCheck, *zAutoCheck, *contAutoCheck;
  QLineEdit *xInterpEdit, *yInterpEdit;
  QCheckBox *legendCheck, *hillBaseCheck, *hillIntersectCheck, *invertHorizCheck, 
    *meshCheck, *labelContCheck;
  QLineEdit *lineOffsetEdit, *lineScaleEdit, *vectorScaleEdit, *rotateAngleEdit, 
    *altAngleEdit, *aziAngleEdit, *lineWidthEdit;
  QComboBox *colourCombo;

  // Variables to store options
  DataPlot *plot;

  // Titles
  QString title, xtitle, ytitle, dtitle;

  // Ranges for plot
  bldata xmin, xmax, xscale;
  bldata ymin, ymax, yscale;
  bldata zmin, zmax, zscale ;
  bldata contmin, contmax, dscale;
  int ncont;

  // Interpolation grid sizes
  int xInterp, yInterp;

  // Various options
  bool legend, hillBase, hillIntersect, invertHoriz, mesh, labelCont;
  int colScheme;
  
};

#endif  // _OPTDLG_H
