//
// xblasvis - C++ version of graphics routines - X11 GUI 
//
// Plot1dDlg : Dialog box to plot data
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <iostream>

#include "plot1ddlg.h"

using namespace std;

//
// Constructor for class
//
Plot1dDlg::Plot1dDlg(QWidget *parent, ReadFile *dfile, DataPlot *p, OptDlg *o, const char *name) : QDialog(parent,name, TRUE) {

  QString text;
  HeaderItem *item;

  // Set up the data file and plot object 
  datafile = dfile;
  plot = p;
  optDlg = o;

  temperature = datafile->GetData("Temp");
  

  // Set up the window
  resize(600,500);

  // Create the tabs
  QWidget *plotPage = new QWidget(this, "plot_tab");
  plotPage->resize(600,550);


  // Details of the 
  if (datafile != NULL) 
    item = datafile->GetFirstHeaderItem();
  else 
    item = NULL;  

  dataBox = new QListBox(plotPage);
  while (item != NULL) {
    if (item->ndim > 0) {
      dataBox->insertItem(item->name);
    }
    item = item->next;    
  }
  
  dataBox->setGeometry(350,50,150,100);

  QLabel *label = new QLabel("Data set",plotPage);
  label->setGeometry(350,20,150,30);

  if (datafile != NULL) 
    item = datafile->GetFirstHeaderItem();
  else 
    item = NULL;  

  QButtonGroup *dimGroup = new QButtonGroup();
  QGroupBox *group = new QGroupBox(plotPage);
  group->setTitle("Dimension to plot");
  group->setGeometry(10,40,310,140);

  QRadioButton *rb = new QRadioButton("x (const z)", group);
  dimGroup->insert(rb,1);
  rb->setGeometry(10,15,140,25);
  rb = new QRadioButton("y (const z)", group);
  dimGroup->insert(rb,2);
  rb->setGeometry(10,45,140,25);
  rb = new QRadioButton("z", group);
  dimGroup->insert(rb,3);
  rb->setGeometry(10,75,140,25);
  rb = new QRadioButton("x (const height)", group);
  dimGroup->insert(rb,14);
  rb->setGeometry(160,15,140,25);
  rb = new QRadioButton("y (const height)", group);
  dimGroup->insert(rb,24);
  rb->setGeometry(160,45,140,25);
  rb = new QRadioButton("x (const T)", group);
  dimGroup->insert(rb,15);
  rb->setGeometry(160,75,140,25);
  rb = new QRadioButton("y (const T)", group);
  dimGroup->insert(rb,25);
  rb->setGeometry(160,105,140,25);
  rb = new QRadioButton("Streamline", group);
  dimGroup->insert(rb,6);
  rb->setGeometry(10,105,140,25);
  plotdim = 1;
  local_plotdim = plotdim;
  dimGroup->setButton(plotdim);
  connect(dimGroup, SIGNAL(clicked(int)), SLOT(dimChanged(int)) );

  label = new QLabel("Value of first dimension", this);
  label->setGeometry(10,190,140,30);
  
  dimVal1Edit = new QLineEdit(plotPage);
  dimVal1Edit->setGeometry(10,220,140,30);
  dimval1 = 0.0;
  dimVal1Edit->setText(text.setNum(dimval1));

  label = new QLabel("Value of second dimension", this);
  label->setGeometry(160,190,160,30);
  
  dimVal2Edit = new QLineEdit(plotPage);
  dimVal2Edit->setGeometry(160,220,140,30);
  dimval2 = 0.0;
  dimVal2Edit->setText(text.setNum(dimval2));


  label = new QLabel("Value of third dimension\n(streamline only)", this);
  label->setGeometry(10,260,160,50);
  
  dimVal3Edit = new QLineEdit(plotPage);
  dimVal3Edit->setGeometry(10,310,140,30);
  dimval3 = 0.0;
  dimVal3Edit->setText(text.setNum(dimval3));

  //Set options
  QPushButton *optButton = new QPushButton("Set options >>",this);
  optButton->setGeometry(380,210,100,30);
  connect(optButton, SIGNAL(clicked()), this, SLOT(optPressed()) );


  QPushButton *plotButton = new QPushButton("Plot",this);
  plotButton->setGeometry(40,450,100,30);
  QPushButton *suspendButton = new QPushButton("Suspend",this);
  suspendButton->setGeometry(180,450,100,30);
  QPushButton *finishButton = new QPushButton("Finish",this);
  finishButton->setGeometry(320,450,100,30);
  QPushButton *cancelButton = new QPushButton("Cancel",this);
  cancelButton->setGeometry(460,450,100,30);      
  
  connect(plotButton, SIGNAL(clicked()), this, SLOT(plotPressed()) );
  connect(suspendButton, SIGNAL(clicked()), this,SLOT(suspendPressed()) );
  connect(finishButton, SIGNAL(clicked()), this,SLOT(finishPressed()) );
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelPressed()) );
    
}


//
// Slot called when Plot is pressed
//
void Plot1dDlg::plotPressed() {
  if (apply()) {
    plotGraph();
  }
}

//
// Slot called when Finsh is pressed
//
void Plot1dDlg::suspendPressed() {
  apply();
  reject();  
}

//
// Slot called when Finsh is pressed
//
void Plot1dDlg::finishPressed() {
  if (apply()) {
    plotGraph();
  }
  accept();  
}

//
// Slot called when Finish is pressed
//
void Plot1dDlg::cancelPressed() {
  reject();
}


//
// Slot called when button to set options is pressed
//
void Plot1dDlg::optPressed() {
  optDlg->exec();
}

//
// Called to check and update the options
//
bool Plot1dDlg::apply() {

  int id;
  double val, val2, val3;
  QString text;
  bool err, err2, err3;

  // Get the data name if there is an item selected
  id = dataBox->currentItem();
  if (id == -1) {
    QMessageBox::warning(this,"Warning!","No data set selected");
    return FALSE;
  }

  text = dimVal1Edit->text();
  val = text.toDouble(&err);
  text = dimVal2Edit->text();
  val2 = text.toDouble(&err2);
  text = dimVal3Edit->text();
  val3 = text.toDouble(&err3);
  if (!err || !err2 || ((local_plotdim == 6) && !err3) ) {
    QMessageBox::warning(this,"Warning!","Incorrect value for dimensions");
    return FALSE;    
  }  
  
  data = dataBox->text(id);
  dimval1 = val;
  dimval2 = val2;
  if (local_plotdim == 6) {
    dimval3 = val3;
  }

  plotdim = local_plotdim;

  return TRUE;
}


//
// Plot a graph
//
void Plot1dDlg::plotGraph() {
  DataSet *d1;
  bldata *xgrid, *ygrid, *zgrid, *sgrid, *hgrid;
  int npts;
  
  
  // Start the plot if necessary
  if (!plot->IsOpen()) {
    startPlot();
  }

  d1 = datafile->GetData(data);

  if (plotdim != 6) {
    plot->LinePlot(d1, plotdim, dimval1, dimval2, "Test", temperature);
  }
  else {
    cout << "Start trajectory calculation..." << endl;
    datafile->GetTraj(dimval1,dimval2,dimval3,-1.0,100,1000, &xgrid, &ygrid, &zgrid, 
		      &sgrid, &hgrid, &npts);
    cout << "Trajectory done! Starting plot..." << endl;
    plot->LinePlot(d1, npts, xgrid, ygrid, zgrid, hgrid, sgrid, "Test");
  }
  
}

// ********************
// * Start a new plot *
// ********************
void Plot1dDlg::startPlot() {

  const char *fileName;
  const char filter[7][30] = {"","","Postscript (*.ps,*.eps)","Postscript (*.ps,*.eps)","XFig (*.fig)","PBM (*.pbm)", "PNG (*.png)"};

  if (plot->IsOpen()) {
    QMessageBox::warning(this,"Warning!","Plot already open");
    return;
  }

  if (plot->NeedDevFile()) {
    fileName = QFileDialog::getSaveFileName(NULL,filter[dev]);
    plot->SetDevFile(fileName);
  }

  plot->InitPlot();

}

//
// Close plot if it is open
//
void Plot1dDlg::finishPlot() {

  if (plot->IsOpen()) {
    plot->FinishPlot();
  }
}

//
// Called when the dimension to plot is changed
//
void Plot1dDlg::dimChanged(int id) {

  local_plotdim = id;

}


//
// Get the name of the data set selected
//
const char *Plot1dDlg::getDataName() {
  return data;
}

//
// Get the surface to plot through
//
int Plot1dDlg::getPlotDim() {
  return plotdim;
}


//
// Get the value of the other dimensions
//
double Plot1dDlg::getPlotDimVal1() {
  return dimval1;
}

//
// Get the value of the other dimensions
//
double Plot1dDlg::getPlotDimVal2() {
  return dimval2;
}

#include "plot1ddlg.moc"
