//
// xblasvis - C++ version of graphics routines - X11 GUI 
//
// PlotDlg : Dialog box to plot data
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

#include "plotdlg.h"

using namespace std;

//
// Constructor for class
//
PlotDlg::PlotDlg(QWidget *parent, ReadFile *dfile, DataPlot *p, OptDlg *o, const char *name) : QDialog(parent,name, TRUE) {

  QString text;
  HeaderItem *item;

  // Set up the data file and plot object 
  datafile = dfile;
  plot = p;
  optDlg = o;

  temperature = datafile->GetData("Temp");
  

  // Set up the window
  resize(600,500);

  // Create the tabs
  QWidget *plotPage = new QWidget(this, "plot_tab");
  plotPage->resize(600,550);


  // Details of the 
  if (datafile != NULL) 
    item = datafile->GetFirstHeaderItem();
  else 
    item = NULL;  

  dataBox = new QListBox(plotPage);
  while (item != NULL) {
    if (item->ndim > 0) {
      dataBox->insertItem(item->name);
    }
    item = item->next;    
  }
  
  dataBox->setGeometry(350,50,150,100);

  QLabel *label = new QLabel("Data set",plotPage);
  label->setGeometry(350,20,150,30);

  if (datafile != NULL) 
    item = datafile->GetFirstHeaderItem();
  else 
    item = NULL;  

  dataBox2 = new QListBox(plotPage);
  while (item != NULL) {
    if (item->ndim > 0) {
      dataBox2->insertItem(item->name);
    }
    item = item->next;    
  }
  
  dataBox2->setGeometry(350,200,150,100);

  label = new QLabel("Second data set",plotPage);
  label->setGeometry(350,170,150,30);

  plotTypeGroup = new QButtonGroup();
  QGroupBox *group = new QGroupBox(plotPage);
  group->setTitle("Plot type");
  group->setGeometry(10,40,310,110);

  QRadioButton *rb = new QRadioButton("Line contour", group);
  plotTypeGroup->insert(rb,1);
  rb->setGeometry(10,15,140,25);
  rb = new QRadioButton("Shaded contour", group);
  plotTypeGroup->insert(rb,2);
  rb->setGeometry(10,45,140,25);
  rb = new QRadioButton("Surface", group);
  plotTypeGroup->insert(rb,3);
  rb->setGeometry(10,75,140,25);
  rb = new QRadioButton("Trajectory", group);
  plotTypeGroup->insert(rb,4);
  rb->setGeometry(160,15,140,25);
  rb = new QRadioButton("Vector", group);
  plotTypeGroup->insert(rb,5);
  rb->setGeometry(160,45,140,25);
  rb = new QRadioButton("Hodograph", group);
  plotTypeGroup->insert(rb,6);
  rb->setGeometry(160,75,140,25);
  plottype = 2;
  local_plottype = plottype;
  plotTypeGroup->setButton(plottype);
  connect(plotTypeGroup, SIGNAL(clicked(int)), SLOT(typeChanged(int)) );

  surfaceGroup = new QButtonGroup();
  group = new QGroupBox(plotPage);
  group->setTitle("Surface to plot");
  group->setGeometry(10,180,310,110);

  rb = new QRadioButton("Const x", group);
  surfaceGroup->insert(rb,1);
  rb->setGeometry(10,15,140,25);
  rb = new QRadioButton("Const y", group);
  surfaceGroup->insert(rb,2);
  rb->setGeometry(10,45,140,25);
  rb = new QRadioButton("Const z", group);
  surfaceGroup->insert(rb,3);
  rb->setGeometry(10,75,140,25);
  rb = new QRadioButton("Const height", group);
  surfaceGroup->insert(rb,4);
  rb->setGeometry(160,15,140,25);
  rb = new QRadioButton("Isentropic", group);
  surfaceGroup->insert(rb,5);
  rb->setGeometry(160,45,140,25);
  rb = new QRadioButton("Const streamfn", group);
  surfaceGroup->insert(rb,6);
  rb->setGeometry(160,75,140,25);
  plotdim = 1;
  local_plotdim = plotdim;
  surfaceGroup->setButton(plotdim);
  connect(surfaceGroup, SIGNAL(clicked(int)), SLOT(dimChanged(int)) );

  label = new QLabel("Value of surface", this);
  label->setGeometry(10,300,140,30);
  
  dimValEdit = new QLineEdit(plotPage);
  dimValEdit->setGeometry(10,330,140,30);
  dimval = 0.0;
  dimValEdit->setText(text.setNum(dimval));

  //Set options
  QPushButton *optButton = new QPushButton("Set options >>",this);
  optButton->setGeometry(450,350,100,30);
  connect(optButton, SIGNAL(clicked()), this, SLOT(optPressed()) );


  QPushButton *plotButton = new QPushButton("Plot",this);
  plotButton->setGeometry(40,450,100,30);
  QPushButton *suspendButton = new QPushButton("Suspend",this);
  suspendButton->setGeometry(180,450,100,30);
  QPushButton *finishButton = new QPushButton("Finish",this);
  finishButton->setGeometry(320,450,100,30);
  QPushButton *cancelButton = new QPushButton("Cancel",this);
  cancelButton->setGeometry(460,450,100,30);      
  
  connect(plotButton, SIGNAL(clicked()), this, SLOT(plotPressed()) );
  connect(suspendButton, SIGNAL(clicked()), this,SLOT(suspendPressed()) );
  connect(finishButton, SIGNAL(clicked()), this,SLOT(finishPressed()) );
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelPressed()) );
    
}


//
// Slot called when Plot is pressed
//
void PlotDlg::plotPressed() {
  apply();
  plotGraph();
}

//
// Slot called when Finsh is pressed
//
void PlotDlg::suspendPressed() {
  apply();
  reject();  
}

//
// Slot called when Finsh is pressed
//
void PlotDlg::finishPressed() {
  apply();
  plotGraph();
  accept();  
}

//
// Slot called when Finish is pressed
//
void PlotDlg::cancelPressed() {
  reject();
}


//
// Slot called when button to set options is pressed
//
void PlotDlg::optPressed() {
  optDlg->exec();
}

//
// Called to check and update the options
//
void PlotDlg::apply() {

  int id, id2 = -1;
  double val;
  QString text;
  bool err;

  // Get the data name if there is an item selected
  id = dataBox->currentItem();
  if (id == -1) {
    QMessageBox::warning(this,"Warning!","No data set selected");
    return;
  }

  // Get the second data set if needed
  if ((local_plottype == 5) || (local_plottype == 6)) {
    id2 = dataBox2->currentItem();
    if (id2 == -1) {
      QMessageBox::warning(this,"Warning!","No second data set selected");
      return;
    }
  }
  
  text = dimValEdit->text();
  val = text.toDouble(&err);

  if (!err) {
    QMessageBox::warning(this,"Warning!","No value entered for surface");
    return;    
  }  

  
  data = dataBox->text(id);
  dimval = val;

  // Only get second data set if needed.
  if (id2 != -1)
    data2 = dataBox2->text(id2);


  plottype = local_plottype;
  plotdim = local_plotdim;
}


//
// Plot a graph
//
void PlotDlg::plotGraph() {
  DataSet *d1, *d2;
  
  // Start the plot if necessary
  if (!plot->IsOpen()) {
    startPlot();
  }

  d1 = datafile->GetData(data);

  switch (plottype) {
  case 1:
    plot->LineContPlot(d1,plotdim,dimval,temperature);
    break;
  case 2:
    plot->ColContPlot(d1,plotdim,dimval,temperature);
    break;
  case 3:
    plot->SurfacePlot(d1,plotdim,dimval,temperature);
    break;
  case 4:
    QMessageBox::warning(this,"Warning!","Not yet implemented");
    break;
  case 5:
    d2 = datafile->GetData(data2);
    plot->VectorPlot(d1,d2,plotdim,dimval,temperature);
    break;
  case 6:
    QMessageBox::warning(this,"Warning!","Not yet implemented");
    break;
  }    
  
}

// ********************
// * Start a new plot *
// ********************
void PlotDlg::startPlot() {

  const char *fileName;
  const char filter[7][30] = {"","","Postscript (*.ps,*.eps)","Postscript (*.ps,*.eps)","XFig (*.fig)","PBM (*.pbm)", "PNG (*.png)"};

  if (plot->IsOpen()) {
    QMessageBox::warning(this,"Warning!","Plot already open");
    return;
  }

  if (plot->NeedDevFile()) {
    fileName = QFileDialog::getSaveFileName(NULL,filter[dev]);
    plot->SetDevFile(fileName);
  }

  plot->InitPlot();

}

//
// Close plot if it is open
//
void PlotDlg::finishPlot() {

  if (plot->IsOpen()) {
    plot->FinishPlot();
  }
}

//
// Called when the plot type is changed
//
void PlotDlg::typeChanged(int id) {

  local_plottype = id;

}

//
// Called when the surface to plot through is changed
//
void PlotDlg::dimChanged(int id) {

  local_plotdim = id;

}


//
// Get the name of the data set selected
//
const char *PlotDlg::getDataName() {
  return data;
}

//
// Get the name of the data set selected
//
const char *PlotDlg::getDataName2() {
  return data2;
}


//
// Get the plot type
//
int PlotDlg::getPlotType() {
  return plottype;
}

//
// Get the surface to plot through
//
int PlotDlg::getPlotDim() {
  return plotdim;
}


//
// Get the value of the surface to plot through
//
double PlotDlg::getPlotDimVal() {
  return dimval;
}

#include "plotdlg.moc"
