// 
// xblasvis - C++ version of graphics routines - X11 GUI  
// 
// PlotDlg : Dialog box for plotting data
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>
//

#ifndef _PLOTDLG_H
#define _PLOTDLG_H

#include <qdialog.h>
#include <qlistbox.h>
#include <qbuttongroup.h>
#include <qlineedit.h>

#include "readfile.h"
#include "dataplot.h"
#include "optdlg.h"

class PlotDlg : public QDialog {
  Q_OBJECT
public:
  PlotDlg(QWidget *, ReadFile *, DataPlot *, OptDlg *, const char * = NULL);  

 public slots:
  void plotPressed();
  void suspendPressed();
  void finishPressed();
  void cancelPressed();
  void optPressed();
  void typeChanged(int);
  void dimChanged(int);

public:
  void finishPlot();
  const char *getDataName();
  const char *getDataName2();
  int getPlotType();
  int getPlotDim();
  double getPlotDimVal();

private:
  void apply();
  void plotGraph();
  void startPlot();

private:
  QButtonGroup *plotTypeGroup;
  QButtonGroup *surfaceGroup;
  QListBox *dataBox, *dataBox2;
  QLineEdit *dimValEdit;

  OptDlg *optDlg;

  // Plotting data
  DataPlot *plot;
  ReadFile *datafile;
  DataSet *temperature;
  // Current output device
  DataPlot::devtype dev;

  // Data to store results   
  const char *data, *data2;  
  int plottype, local_plottype;
  int plotdim, local_plotdim;
  double dimval;

};

#endif  // _PLOTDLG_H
