//
// ArrayUtil: Utility class containing utilities to allocate and free arrays
// Inherited by other classes
//

#ifndef _ARRAYUTIL_H
#define _ARRAYUTIL_H

class ArrayUtil {
 protected:
  // Create 3D array of given size
  template <class T> T***ArrayUtil::Alloc3DArray(int x, int y, int z) {
    T ***tmp;

    tmp = new (T **)[x];
    tmp[0] = new (T*)[x*y];
    tmp[0][0] = new T[x*y*z];  
    for (int j=1;j<y;j++) {
      tmp[0][j] = tmp[0][j-1] + z;
    }
    for (int i = 1;i<x;i++) {
      tmp[i] = tmp[i-1] + y;
      tmp[i][0] = tmp[i-1][0] + y*z;
      for (int j=1;j<y;j++) {
	tmp[i][j] = tmp[i][j-1] + z;
      }
    }
  
    return tmp;
  }

  // Create array of given size
  template <class T> T** AllocArray(int n1, int n2) {
    T **array; 
  
    array = new (T*)[n1];
    array[0] = new T[n1*n2];
    for (int i=1;i<n1;i++) {
      array[i] = array[i-1] + n2;
    }

    return array;
  }

  template <class T> void FreeArray(T** array) {
    delete[] array[0];
    delete[] array;  
  }

  template <class T> void Free3DArray(T*** array) {
    delete[] array[0][0];
    delete[] array[0];
    delete[] array;  
  }


  //
  // Get minimum of an vector
  //
  template <class T> T GetMin(T * d, int nx) {
    T min = d[0];
    
    for (int i=0;i<nx;i++) {
      min = (d[i]<min)?d[i]:min;
    }
    
    return min;
  }

  //
  // Get maximum of an vector
  //
  template <class T> T GetMax(T * d, int nx) {
    T max = d[0];
    
    for (int i=0;i<nx;i++) {
      max = (d[i]>max)?d[i]:max;     
    }
    
    return max;
  }

  //
  // Get minimum of an array
  //
  template <class T> T GetMin(T ** d, int nx, int ny) {
    T min = d[0][0];
    
    for (int i=0;i<nx;i++) {
      for (int j=0;j<ny;j++) {
	min = (d[i][j]<min)?d[i][j]:min;
      }
    }
    
    return min;
  }

  //
  // Get maximum of an array
  //
  template <class T> T GetMax(T **d, int nx, int ny) {
    T max = d[0][0];
    
    for (int i=0;i<nx;i++) {
      for (int j=0;j<ny;j++) {
	max = (d[i][j]>max)?d[i][j]:max;
      }
    }
    
    return max;
  }  

  //
  // Get minimum of an array
  //
  template <class T> T GetMinNonzero(T ** d, int nx, int ny) {
    T min;

    if (d[0][0] != 0) {
      min = d[0][0];
    }
    else {
      min = 1.0E20;
    }
    
    
    for (int i=0;i<nx;i++) {
      for (int j=0;j<ny;j++) {
	if (d[i][j] != 0) {
	  min = (d[i][j]<min)?d[i][j]:min;
	}
      }
    }
    
    return min;
  }

  //
  // Get maximum of an array
  //
  template <class T> T GetMaxNonzero(T **d, int nx, int ny) {
    T max;

    if (d[0][0] != 0) {
      max = d[0][0];
    }
    else {
      max = -1.0E20;
    }
    
    for (int i=0;i<nx;i++) {
      for (int j=0;j<ny;j++) {
	if (d[i][j] != 0) {
	  max = (d[i][j]>max)?d[i][j]:max;
	}
      }
    }
    
    return max;
  }

};



#endif
