// -*-c++-*-
// 
// blasvis++ - C++ version of graphics routines
//
// BlasFile : Class to read the information in from the
//            Blasius file.
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>

#ifndef _BLASFILE_H
#define _BLASFILE_H

#include "readfile.h"
#include "blastype.h"
#include "dataset.h"
#include "timeseries.h"
#include "datavect.h"
#include "zstream.h"

class BlasFile : public ReadFile {
 public:
  BlasFile();
  BlasFile(char*);
  ~BlasFile();
  
  virtual void CloseFile();
  virtual void DisplayParms();

  DataSet *GetData(const char[]);
  TimeSeries *GetTimeSeries(const char[]);
  bldata GetParm(const char[]);
  virtual DataSet *GetPressureCorr();
  virtual DataSet *GetSpeedup();
  virtual DataVect *GetTau();
  virtual DataSet *GetPVSource();
  virtual DataSet *GetLESuw();
  virtual DataSet *GetLESuu();
  virtual DataSet *GetLESvv();
  virtual DataSet *GetLESww();
  virtual DataSet *GetLESenergy();

 private:
  void InitFile();
  void ReadHeader();
  void ReadParms();
  void LoadGrids();
  void ReadArray(bldata *, int, HeaderItem *, long &);

  // Methods to read / unread characters from header
  char ReadChar() {
    char inchar;
    //    file >> inchar;
    file.get(inchar);
    crec++;
    if (!norec && (crec == irecl)) {
      crec = 0;
      hdrrecs++;
      file.seekg(2*reclen,std::ios::cur);
    }

    return inchar;
  }
  
  void UnreadChar() {
    file.unget();
    crec--;
    if (crec == -1) {
      crec = irecl-1;
      hdrrecs--;
    }
  }

  // Private data for class


  // Pointers for calulated quantities 
  //  DataSet *pv, *pvalt, *bernoulli, *windspeed, *speedup, *deltaT, *pcorr,
  //    *timestep, *bldepth;
  //  DataVect *pvflux,*pvfluxalt, *gradt;
  DataVect *tau;

  // Staggered orography grids
  DataSet *orogx, *orogy, *orogxy;

  // Variables for reading header
  int hdrrecs;
  long crec;

  // Maximum length of name labels in header
  // (including terminating '\0')
  static const int hnlen = 11;

};

#endif

