// -*-c++-*-
// 
// blasvis++ - C++ version of graphics routines
//
// DataSet : Class to encapsulate an array of data, including
//           reading in the data and interpolating it onto an
//           an arbitrary grid.
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>

#ifndef _DATASET_H
#define _DATASET_H

#include "blastype.h"
#include "arrayutil.h"

#ifndef NULL
#define NULL 0
#endif

//#define HACK 

class DataVect;

struct Grid {
  bldata *xgrid;
  bldata *ygrid;
  bldata *zgrid;
  int ndim,nx,ny,nz;  
};

class DataSet : public ArrayUtil {
 public: 
  DataSet();
  DataSet(int, int, int, int);
  DataSet(bldata***,int,int,int,int, bldata * = NULL, bldata * = NULL, 
	  bldata * = NULL, bldata = 0, bldata ** = NULL );

  void SetGrids(bldata*,bldata*,bldata*, int, int, int, bldata, DataSet * = NULL);
  void SetGrids(bldata*,bldata*,bldata*, int, int, int, bldata, bldata **);
  void DisplayData();
  void DisplayData(const char *);
  void DisplayGrids();
  void DisplaySummary();
  //  bldata *CreateInterpGrid(int, bldata, bldata, int, bldata, bool);
  void CreateInterpGrid(int, int, bldata, bldata, bldata, bldata, bldata,
			bldata, int, bldata **, bldata **,
			bldata ***, bldata ***, 
			bldata ***, bldata ***, bldata, bldata, bldata, 
			bldata, bool, bool = true, DataSet * = NULL);
  void GetInterpSection(bldata ***, bldata **, bldata **, bldata **, 
			bldata **, bldata, int, int, bool, bool = true);
  void CreateInterpGrid1D(int, bldata, bldata, bldata, bldata, bldata,
			bldata, int, bldata **, 
			bldata **, bldata **, 
			bldata **, bldata **, bldata, bldata, bldata, 
			bldata, bool, bool = true, DataSet * = NULL);
  void GetInterpLine(bldata **, bldata *, bldata *, bldata *,
			bldata *, bldata, int, bool, bool = true);
  void GetInterpLine2(bldata **, bldata *, bldata *, bldata *,
			bldata *, bldata, int, bool);
  void GetInterpLine_old(bldata **, int, bldata, bldata, int, bldata, bldata, 
		     bldata = 0, bldata = 1, bool = true, DataSet * = NULL);
  // Convert temperatures to Kelvin
  void ConvTemp(bldata, bldata, bool);
  // Swap around arrays for orography
  void SwapData();
  // Mark the vector dimension of the data if it is a vector
  void SetIsVector(int);
  int GetIsVector();

  // Get the normalised data
  DataSet *Normalise();

  bldata *Get1D(int);
  bldata **Get2D(int);
  int GetDim(int);
  int GetDims();
  bldata *** GetData();
  bldata GetVal(bldata,bldata,bldata);
  bldata GetVal2(bldata,bldata,bldata, bool = false);
  bldata Get1DInterp(bldata, int, bldata *, bldata *);
  bldata GetGridMin(int);
  bldata GetGridMax(int);
  bldata *GetGrid(int);
  bldata **GetOrog();
  char* GetName();
  void SetName(const char*);

  void GetDataWeight(int, int, bldata, bldata, bldata, bldata, bldata,int &,
		     bldata & ,bldata &);

  DataVect *Grad();

  friend class DataVect;

  // Create an independent copy of the dataset which can be altered.
  DataSet * copy();

  // Overloaded operators
  template <class T> DataSet operator *= (T val) {
    for (int k=0;k<dim[2];k++) {
      for (int j=0;j<dim[1];j++) {
	for (int i=0;i<dim[0];i++) {
	  data[i][j][k] *= val;
	}
      }
    }
    return *this;    
  };
  DataSet operator += (DataSet val) {
    for (int k=0;k<dim[2];k++) {
      for (int j=0;j<dim[1];j++) {
	for (int i=0;i<dim[0];i++) {
	  data[i][j][k] += val.data[i][j][k];
	}
      }
    }
    return *this;
  };
  DataSet operator -= (DataSet val) {
    for (int k=0;k<dim[2];k++) {
      for (int j=0;j<dim[1];j++) {
	for (int i=0;i<dim[0];i++) {
	  data[i][j][k] -= val.data[i][j][k];
	}
      }
    }
    return *this;
  };
  template <class T> DataSet operator * (T val) {
    DataSet *tmp = copy();
    *tmp *= val;
    return *tmp;
  }

  DataSet operator + (DataSet val) {
    DataSet *tmp = copy();
    *tmp += val;
    return *tmp;
  }

  DataSet operator - (DataSet val) {
    DataSet *tmp = copy();
    *tmp -= val;
    return *tmp;
  }


private:
  void GetGridWeight(bldata *,int,bldata,int &,bldata & ,bldata &, bldata);
  int GetIndex(bldata*, int, bldata);
  int GetOrder(int, int, int &);
  
private:
  bldata ***data;
  bldata *xgrid,*ygrid,*zgrid;
  bldata **orog;
  bldata H;
  int ndim;
  int dim[3];
  // Name of data set
  char name[20];

  int isvector;

};

#endif
