// -*-c++-*-
// 
// escstring - String class with expansion of escape commands
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>

#include "escstring.h"
#include <cstring>
#include <iostream>

using namespace std;

//
// Constructor
//
EscString::EscString() {
  first = NULL;
  last = NULL;
  rawtext = NULL;
  esc = '%';
}


//
// Destructor
//
EscString::~EscString() {
  EscList *c,*n;

  // Delete the list of escape keys
  c = first;
  while (c != NULL) {
    n = c->next;
    delete c;
    c = n;
  }
  
}

//
// Set the text string
//
void EscString::SetString(char *str) {
  int len;

  if (rawtext != NULL) {
    delete[] rawtext;
  }
  
  len = strlen(str)+1;
  rawtext = new char[len*sizeof(char)];
  strncpy(rawtext,str,len);
}

// 
// Get the text string
//
char * EscString::GetString() {
  return rawtext;
}

// 
// Get the expanded text string
//
char * EscString::GetExpandedString() {
  char *exptext, *temp;
  char *sptr, *dptr;
  EscList *ptr;
  int nchar, nmax, nesc, tlen, loc;

  nesc = 0;
  sptr = rawtext;
  while (*sptr != '\0') {
    if (*sptr == esc) nesc++;
    sptr++;
  }

  // Guess how big a string we need to start with
  nmax = strlen(rawtext) + nesc*10 + 1;
  
  exptext = new char[nmax];

  sptr = rawtext;
  dptr = exptext;

  nchar = 0;

  while (*sptr != '\0') {
    while ((*sptr != '\0')&&(nchar < nmax)) {
      // Just plain text then copy it
      if (*sptr != esc) {
	*dptr = *sptr;
	sptr++;
	dptr++;
	nchar++;
      }
      // Expand any escape characters
      else {
	sptr++;
	// Special case for escape character
	if (*sptr == esc) {
	  *dptr = esc;
	  dptr++;
	  nchar++;      
	}
	else {
	  ptr = FindKey(*sptr);
	  if (ptr == NULL) {
	    cerr << "Invalid escape sequence ignored " << endl;
	  }
	  else {
	    tlen = strlen(ptr->text);
	    if (tlen + nchar > nmax) {
	      sptr--;
	      break;
	    }
	    // OK as we know the string is long enough
	    strcpy(dptr,ptr->text);
	    nchar+= tlen;
	    dptr += tlen;
	  }
	  sptr++;
	}
      }
    }
    if (*sptr != '\0') {
      // Need to extend the string
      temp = new char[nmax+10];
      loc = (int) (dptr-exptext);
      strncpy(temp,exptext,loc);
      dptr = temp+loc;
      delete[] exptext;
      exptext = temp;
    }
  }
  *dptr = '\0';
    
  return exptext;
}

//
// Set the escape character
//
void EscString::SetEscChar(char e) {
  esc = e;
}

//
// Get the escape character
//
char EscString::GetEscChar() {
  return esc;
}

//
// Set the escape key `key' and associated text `txt'
//
void EscString::SetEscKey(char key, char* text) {
  EscList *ptr;
  int len;

  ptr = FindKey(key);

  if (ptr == NULL) {
    ptr = new EscList;
    ptr->prev = last;
    ptr->next = NULL;
    if (last){
      last->next = ptr;
    }
    else {
      first = ptr;
    }
    last = ptr;
    ptr->key = key;
  }
  else {
    delete ptr->text;
  }

  len = strlen(text)+1;
  ptr->text = new char[len*sizeof(char)];
  strncpy(ptr->text,text,len);

}

//
// Delete an escape key from the list
//
void EscString::ClearEscKey(char key) {
  EscList *ptr, *n, *p;

  ptr = FindKey(key);

  if (ptr == NULL) return;

  p = ptr->prev;
  n = ptr->next;

  if (ptr == first) {
    first = n;
  }
  else {
    p->next = n;
  }
  
  if (ptr == last) {
    last = p;
  }
  else {
    n->prev = p;
  }
  
  delete ptr->text;
  delete ptr;
 
}

//
// Find the EscList item corresponding to a key
//
EscList * EscString::FindKey(char key) {
  EscList *ptr;

  ptr = first;

  while ((ptr != NULL) && (ptr->key != key)) {
    ptr = ptr->next;
  }

  return ptr;
}
