//
// blasvis - C++ version of graphics routine
//
// TimeSeries : Class to encapsulate a time series of data, 
//              including interpolating it onto an arbitrary grid. 
//
// (C) 2004 Andrew Ross <aross@env.leeds.ac.uk>
//

#include "timeseries.h"
#include <iostream>
#include <fstream>
#include <cstring>

using namespace std;

TimeSeries::TimeSeries() {
  data = NULL;
  time = NULL;
  npts = 0;
  name[0] = '\0';
}

TimeSeries::TimeSeries(bldata *d, bldata *t, int n) {
  data = d;
  time = t;
  npts = n;
  name[0] = '\0';
}

//
// Get name of data set
//
char * 
TimeSeries::GetName() {
  return name;
}


//
// Set name of data set
//
void
TimeSeries::SetName(const char *n) {
  name[19] = '\0';
  strncpy(name,n,19);
}

// 
// Get the time at a specific index
//
bldata
TimeSeries::GetTime(int i) {
  if ((i<0) || (i>=npts)) {
    cerr << "Index out of range" << endl;
    return 0;
  }
  return time[i];
}

// 
// Get the value at a specific index
//
bldata
TimeSeries::GetVal(int i) {
  if ((i<0) || (i>=npts)) {
    cerr << "Index out of range" << endl;
    return 0;
  }
  return data[i];
}

//
// Get the value of the data at a specified time
//
bldata
TimeSeries::GetVal(bldata t) {
  int i;
  bldata val;

  if ((t < time[0]) || (t > time[npts-1])) {
    cerr << "Time out of range" << endl;
  }
  
  i = npts-1;

  while (time[i]>t) i--;

  // Linear interpolation of value
  if (i == npts-1) {
    val = data[i];
  }
  else {
    val = (data[i+1]*(t-time[i])+data[i]*(time[i+1]-t))/(time[i+1]-time[i]);
  }

  return val;
  
}

//
// Display time series
//
void
TimeSeries::DisplayData() {

  for (int i=0;i<npts;i++) {
    cout << time[i] << " " << data[i] << endl;
  }
}

//
// Dump time series to file
//
void
TimeSeries::DisplayData(const char *fname) {

  ofstream fout;

  fout.open(fname);

  for (int i=0;i<npts;i++) {
    fout << time[i] << "\t" << data[i] << endl;
  }
}

//
// Get minimum time 
//
bldata
TimeSeries::GetMinTime() {
  return time[0];
}

//
// Get maximum time 
//
bldata
TimeSeries::GetMaxTime() {
  return time[npts-1];
}
