// -*-c++-*-
// 
// blasvis++ - C++ version of graphics routines
//
// VomFile : Class to read the information in from a
//           3dnvom1.6 restart file.
//
// (C) 2001 Andrew Ross <aross@env.leeds.ac.uk>

#ifndef _VOMFILE_H
#define _VOMFILE_H

#include "readfile.h"
#include "blastype.h"
#include "dataset.h"
#include "datavect.h"
#include "zstream.h"
#include "models.h"


class VomFile : public ReadFile {
 public:
  VomFile();
  VomFile(char*);
  ~VomFile();
  
  virtual void CloseFile();
  virtual void DisplayParms();

  DataSet *GetData(const char[]);
  bldata GetParm(const char[]);
  virtual DataSet *GetPressureCorr();
  virtual DataVect *GetTau();
  virtual DataSet *GetPVSource() { return NULL; };
  void GetVorticity();
  void CalcTau();
  virtual DataSet *GetLESuw() { return NULL; };
  virtual DataSet *GetLESuu() { return NULL; };
  virtual DataSet *GetLESvv() { return NULL; };
  virtual DataSet *GetLESww() { return NULL; };
  virtual DataSet *GetLESenergy() { return NULL; };

 private:
  void InitFile();
  void ReadData();
  bldata ***GetArray(int,int,int);
  bldata *GetVector(int);
  bldata GetVal();
  void SkipArray(int,int,int,int);
  void CreateGrids();

  // Private data for class

  // Pointers for quantities read in
  DataSet *u, *v, *w, *temp, *pressure, *rho, *theta0, *visco;
  DataSet *ui, *vi, *uprime, *vprime;
  DataSet *tau11, *tau22, *tau33, *tau12, *tau13, *tau23;
  double fcor, dx, dy, dt, time, nstep, z0;

  // Pointers for calculated quantities 
  DataSet *vortx, *vorty, *vortz;


};

#endif

