//
// izstream - ifstream wrapper to uncompress a file first if necessary.
//

#include "zstream.h"
#include <cstring>
#include <iostream>

#ifdef USE_ZLIB
#include <zlib.h>
#endif

using namespace std;

#ifdef HAS_NEW_IOSTREAM
izstream::izstream(const char *name, ios_base::openmode mode) {
#else
izstream::izstream(const char *name, int mode) {
#endif
  open(name, mode);
}

#ifdef HAS_NEW_IOSTREAM
void izstream::open(const char *name, ios_base::openmode mode) {
#else
void izstream::open(const char *name, int mode) {
#endif
  int ptr;

  // Check if file is compressed - if not the just open it.

  ptr = strlen(name) - 3;
  if (strncmp(name+ptr,".gz",3)) {
     //ifstream::open(name,mode,prot);
     ifstream::open(name,mode);
  }
  else {
    // cout << "Uncompressing file" << endl;
    uncompress(name);
  }
  seekg(0, ios::beg);
}

// Check for zlib support
#ifdef USE_ZLIB
void izstream::uncompress(const char *name) {
  FILE *tmpfd;
  gzFile gzfd;
  static int buflen = 1000;
  char buf[buflen];
  int nread;
#ifdef HAS_NEW_IOSTREAM
  char tmpfname[L_tmpnam];
#endif

// tmpfile is better but new iostream has no attach. Doh!
#ifdef HAS_NEW_IOSTREAM
  tmpnam(tmpfname);
  tmpfd = fopen(tmpfname,"w");
#else
  tmpfd = tmpfile();
  // Check temporary file created ok.
#endif
  if (tmpfd == NULL) return;

  // Open .gz file and check it succeeded
  gzfd = gzopen(name,"rb");
  if (gzfd == NULL) return;

  // Read in data and write to temporary file
  while ( (nread = gzread(gzfd,buf,buflen)) > 0) {
    fwrite(buf,sizeof(char),nread,tmpfd);
  }

  gzclose(gzfd);

#ifdef HAS_NEW_IOSTREAM
  ifstream::open(tmpfname);
#else
  rewind(tmpfd);
  ifstream::attach(fileno(tmpfd));
#endif


  return;
} 
#else
void izstream::uncompress(const char *name) {
   cerr << "Support for compressed files not available" << endl;
}
#endif
